/////////////////////////////////////////////////////////////////////////
////                     fouzouli.C                                  ////
////                                                                 ////
////  Programme de Fouzouli 										 ////
////                 18F252 quartz 4 MHz							 ////
////  Auteur: 	Fribottes					     					 ////
////  Date:	 22 novembre 2003                                     	 ////
////  Utilise le bootloader, mais marche sans 						 ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 2002 Fribotte 		             		 ////
////								     							 ////
/////////////////////////////////////////////////////////////////////////


#include <delays.h>
#include <timers.h>
#include <pwm.h>
#include <adc.h>
#include <usart.h>  
#include <p18f252.h>
#define uchar unsigned char

#include "musique.h"


struct bitArrayStruct  {
  unsigned Bit0:1;
  unsigned Bit1:1;
  unsigned Bit2:1;
  unsigned Bit3:1;
  unsigned Bit4:1;
  unsigned Bit5:1;
  unsigned Bit6:1;
  unsigned Bit7:1;
};

union Slong {
	long 			l;
	unsigned char 	c[4];
	unsigned int	i[2];
	struct bitArrayStruct b[4];
};


typedef union charbitArray{
	unsigned char value;
	struct bitArrayStruct bits;
} charbitArray;

/* IO */

/* Servo */
#define IO_SERVO1 PORTBbits.RB7
#define IO_SERVO1_TRIS TRISBbits.TRISB7
#define IO_SERVO2 PORTBbits.RB6
#define IO_SERVO2_TRIS TRISBbits.TRISB6

#define IO_SERVOD PORTBbits.RB2
#define IO_SERVOD_TRIS TRISBbits.TRISB2
#define IO_SERVOG PORTBbits.RB1
#define IO_SERVOG_TRIS TRISBbits.TRISB1


/* LED de debug */
#define IO_LED PORTBbits.RB5
#define IO_LED_TRIS TRISBbits.TRISB5

/* LED de couleurs */
#define IO_LED_ROUGE PORTCbits.RC4
#define IO_LED_ROUGE_TRIS TRISCbits.TRISC4
#define IO_LED_BLANCHE PORTCbits.RC5
#define IO_LED_BLANCHE_TRIS TRISCbits.TRISC5

/* HP */
#define IO_HP_PLUS PORTCbits.RC3
#define IO_HP_PLUS_TRIS TRISCbits.TRISC3
#define IO_HP_MOINS PORTCbits.RC0
#define IO_HP_MOINS_TRIS TRISCbits.TRISC0

/* detetion IR */
#define IO_IR_AVANT PORTAbits.RA3
#define IO_IR_AVANT_TRIS TRISAbits.TRISA3
#define IO_IR_ARRIERE PORTAbits.RA4
#define IO_IR_ARRIERE_TRIS TRISAbits.TRISA4
#define IO_IR_DROITE PORTAbits.RA1
#define IO_IR_DROITE_TRIS TRISAbits.TRISA1
#define IO_IR_GAUCHE PORTAbits.RA2
#define IO_IR_GAUCHE_TRIS TRISAbits.TRISA2

/* choc */
#define IO_CHOC PORTBbits.RB0
#define IO_CHOC_TRIS TRISBbits.TRISB0



/* Var on IO */
volatile unsigned int servo1_pos;
volatile unsigned int servo2_pos;
volatile unsigned int servog_pos;
volatile unsigned int servod_pos;
volatile uchar servoStatus;

volatile uchar ANStatus;
volatile char readADCOk;

volatile uchar cmpLed = 0;
volatile uchar ledCRouge = 0;
volatile uchar ledCBlanche = 0;
volatile uchar ledCRougeCycle = 0;
volatile uchar ledCBlancheCycle = 0;

/* main compteur */
volatile uchar mainCompteur;
volatile uchar mainCompteurLong;
volatile uchar newhalfms;
uchar mode_choc = 0;
int cmpChoc = 0;
int cmpIr = 0;
uchar mvt_ir = 0;
uchar mouvement_ir = 0;
int cmp_mouvement_ir = 0;

/* musique */
unsigned char cmpChanson = 0;
unsigned int indexMusik = 0;
unsigned char delayMusik50ms = 0;
unsigned char cmpMusikLow = 0;

/* pour le main uniquement */
uchar distance;
uchar typeAvance = 0;
uchar oldTypeAvance = 0;
int cmpTypeAvance =0;

uchar stateTourelle = 0;
int cmpStateTourelle = 0;

uchar distanceArray[8];
uchar distanceNew;
uchar cmpAd;
unsigned int sum;
unsigned int sum2;
int min, max;
uchar random;
uchar choix_choc;
int cmpGir = 0;

uchar modeTourne = 0;
int cmpLedTourne = 0;
unsigned int cmpTourne = 0;
unsigned int cmpStartTourne = 0;

/* servo Positon */
#define SERVO_M90 550 /* tesed */
#define SERVO_M45 960 /* tesed */
#define SERVO_MIDLE 1375 /* tested */
#define SERVO_P45 1790 /* tesed */
#define SERVO_P90 2200 /* tesed */

/* fonctions */
void MyInterrupt(void);

/* timer 0 */
#define TIMER0_RESTART 39

#define MESSAGE_OK 0
#define MESSAGE_ERROR 255

/* Tableau pour les valeurs IR */
/* donne des 1/2 cm */
rom unsigned char valueIR[] = {
 	 255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,
	 255 , 255 , 255 , 255 , 255 , 244 , 225 , 208 ,
	 194 , 182 , 171 , 161 , 152 , 144 , 137 , 130 ,
	 124 , 119 , 114 , 109 , 105 , 101 , 97 , 94 ,
	 91 , 88 , 85 , 82 , 80 , 77 , 75 , 73 ,
	 71 , 69 , 67 , 65 , 64 , 62 , 61 , 59 ,
	 58 , 57 , 55 , 54 , 53 , 52 , 51 , 50 ,
	 49 , 48 , 47 , 46 , 45 , 45 , 44 , 43 ,
	 42 , 42 , 41 , 40 , 40 , 39 , 38 , 38 ,
 	 37 , 37 , 36 , 35 , 35 , 34 , 34 , 34 ,
	 33 , 33 , 32 , 32 , 31 , 31 , 31 , 30 ,
	 30 , 29 , 29 , 29 , 28 , 28 , 28 , 27 ,
	 27 , 27 , 26 , 26 , 26 , 26 , 25 , 25 ,
 	 25 , 25 , 24 , 24 , 24 , 24 , 23 , 23 ,
	 23 , 23 , 22 , 22 , 22 , 22 , 22 , 21 ,
	 21 , 21 , 21 , 21 , 20 , 20 , 20 
 };




static void robotAvance(void)
{
	servog_pos = SERVO_M90;
	servod_pos = SERVO_P90;
}

static void robotRecule(void)
{
	servog_pos = SERVO_P90;
	servod_pos = SERVO_M90;
}

static void robotStop(void)
{
	servog_pos = 0;
	servod_pos = 0;
}

static void robotGauche(void)
{
	servog_pos = 0;
	servod_pos = SERVO_P90;
}

static void robotGaucheFort(void)
{
	servog_pos = SERVO_P90;
	servod_pos = SERVO_P90;
}

static void robotDroite(void)
{
	servog_pos = SERVO_M90;
	servod_pos = 0;
}

static void robotDroiteFort(void)
{
	servog_pos = SERVO_M90;
	servod_pos = SERVO_M90;
}

/* use it only without 2-prioriy interrupt level */
static void DisableAllInterrupt(void)
{
	INTCONbits.GIE = 0;
	INTCONbits.PEIE = 1;
}

/* use it only without 2-prioriy interrupt level */
static void EnableAllInterrupt(void)
{
	INTCONbits.GIE = 1;
	INTCONbits.PEIE = 1;
}


void setServo(char numero) 
{
	unsigned int position;
	
	// set output to one, and set value
	if (numero == 1) {
		if (servo1_pos!=0) {
			IO_SERVO1 = 1;
			position = 65535 - servo1_pos;
			WriteTimer3(position);
		}
		else
			WriteTimer3(65535-SERVO_MIDLE);
		
	}
	if (numero == 2) {
		if (servo2_pos!=0) {
			IO_SERVO2 = 1;
			position = 65535 - servo2_pos;
			WriteTimer3(position);
		} 
		else 
			WriteTimer3(65535-SERVO_MIDLE);
	}
	if (numero == 3) {
		if (servog_pos!=0) {
			IO_SERVOG = 1;
			position = 65535 - servog_pos;
			WriteTimer3(position);
		}
		else
			WriteTimer3(65535-SERVO_MIDLE);
	}
	if (numero == 4) {
		if (servod_pos!=0) {
			IO_SERVOD = 1;
			position = 65535 - servod_pos;
			WriteTimer3(position);
		}
		else 
			WriteTimer3(65535-SERVO_MIDLE);
	}
	// Start timer 3 interrupt
	PIE2bits.TMR3IE = 1;
}

void endServo(char numero) 
{
	// end timer 3 interrupt
	PIE2bits.TMR3IE = 0;

	if (numero == 1) {
		IO_SERVO1 = 0;
	}
	if (numero == 2) {
		IO_SERVO2 = 0;
	}
	if (numero == 3) {
		IO_SERVOG = 0;
	}
	if (numero == 4) {
		IO_SERVOD = 0;
	}
}

// Debug avec bootloader
#pragma code start=0x000
void atStart(void)
{
_asm GOTO 0x200 _endasm
}
#pragma code // retour  la zone de code

#pragma code interrupt=0x008
void atInterrupt(void)
{
_asm GOTO 0x208 _endasm
}
#pragma code // retour  la zone de code




// ************************
// ****  Interruptions ****
// ************************
#pragma code highVector=0x208
void atHighVector(void)
{
_asm GOTO MyInterrupt _endasm
}
#pragma code // retour  la zone de code

/* Compilo error : 
	do not use * in the interrupt ! as the compilo use PRODL and PRODH for return of int f() fonctions !
	do not call int f() in the interrupt for the same reason

*/

#pragma interrupt MyInterrupt 
void MyInterrupt(void)
{
	uchar sauv1;
	uchar sauv2;

	sauv1 = PRODL;
	sauv2 = PRODH;

	// Timer 0 interrupt, every 0.5ms
	if (INTCONbits.TMR0IF) {
		newhalfms = 1;

		ANStatus++;
		if (ANStatus == 1) {
			/* convert already started */
			if (BusyADC())
				ANStatus = 0;
			else {
				readADCOk = 1;
			}		
		}
		if (ANStatus == 2) {
			/* start convert */
			if (readADCOk==1)
				ANStatus--;				
			else {
				ConvertADC();
				ANStatus = 0;
			}
		}

		WriteTimer0(TIMER0_RESTART);
		INTCONbits.TMR0IF = 0;
		mainCompteur++;

		//testIR();

		if (mainCompteur == 40) {
			mainCompteurLong++;
			/*
			if (IO_LED)
				IO_LED = 0;
			else
				IO_LED = 1;
			*/
			mainCompteur = 0;
			servoStatus = 0;
			setServo(1);
		}

		/* leds */
		cmpLed++;
		if (cmpLed>10) {
			ledCRougeCycle = ledCRouge;
			ledCBlancheCycle = ledCBlanche;
			cmpLed = 0;
			if (ledCRougeCycle!=0) {
				IO_LED_ROUGE = 1;
			}
			if (ledCBlancheCycle!=0) {
				IO_LED_BLANCHE = 1;
			}		
		}
		if (cmpLed == ledCRougeCycle) {
				IO_LED_ROUGE = 0;		
		}
		if (cmpLed == ledCBlancheCycle) {
				IO_LED_BLANCHE = 0;
		}

		//testIR();


	}

	// timer 2 interrupt, every 0.5ms
	// Reset the interruption
	if (PIR1bits.TMR2IF) 
		PIR1bits.TMR2IF = 0;

	// timer3 interrupt, for servo
	if (PIR2bits.TMR3IF) {
		PIR2bits.TMR3IF = 0;

		switch (servoStatus) 
		{
			case 0:			
				endServo(1);
				setServo(2);
				servoStatus++;
			break;
			case 1:
				endServo(2);
				setServo(3);
				servoStatus++;
			break;
			case 2:
				endServo(3);
				setServo(4);
				servoStatus++;
			break;
			case 3:
				endServo(4);
				servoStatus++;
			break;
			default:
			break;
		}
	}

	PRODL = sauv1;
	PRODH = sauv2;
		

}



void musique(void)
{
	uchar t1,t2;

	if (indexMusik&1) {
			IO_HP_PLUS = 0;	
			SetDCPWM2(0);
			delayMusik50ms = 1;
			indexMusik++;
			return;			
	}
		
	if (cmpChanson == 0) {
		t1 = note_list_0[indexMusik];
		t2 = note_list_0[(indexMusik)+1];
	}
	if (cmpChanson == 1) {
		t1 = note_list_1[indexMusik];
		t2 = note_list_1[(indexMusik)+1];
	}
	if (cmpChanson == 2) {
		t1 = note_list_2[indexMusik];
		t2 = note_list_2[(indexMusik)+1];
	}
	if (cmpChanson == 3) {
		t1 = note_list_3[indexMusik];
		t2 = note_list_3[(indexMusik)+1];
	}
	if (cmpChanson == 4) {
		t1 = note_list_4[indexMusik];
		t2 = note_list_4[(indexMusik)+1];
	}
	if (cmpChanson == 5) {
		t1 = note_list_5[indexMusik];
		t2 = note_list_5[(indexMusik)+1];
	}
	if (cmpChanson == 6) {
		t1 = note_list_6[indexMusik];
		t2 = note_list_6[(indexMusik)+1];
	}
	if (cmpChanson == 7) {
		t1 = note_list_7[indexMusik];
		t2 = note_list_7[(indexMusik)+1];
	}
	if (cmpChanson == 8) {
		t1 = note_list_8[indexMusik];
		t2 = note_list_8[(indexMusik)+1];
	}
	if (cmpChanson == 9) {
		t1 = note_list_9[indexMusik];
		t2 = note_list_9[(indexMusik)+1];
	}
	if (cmpChanson == 10) {
		t1 = note_list_10[indexMusik];
		t2 = note_list_10[(indexMusik)+1];
	}
	if (cmpChanson == 11) {
		t1 = note_list_11[indexMusik];
		t2 = note_list_11[(indexMusik)+1];
	}
	if (cmpChanson == 12) {
		t1 = note_list_12[indexMusik];
		t2 = note_list_12[(indexMusik)+1];
	}
	if (cmpChanson == 13) {
		t1 = note_list_13[indexMusik];
		t2 = note_list_13[(indexMusik)+1];
	}
	if (cmpChanson == 14) {
		t1 = note_list_14[indexMusik];
		t2 = note_list_14[(indexMusik)+1];
	}
	if (cmpChanson == 15) {
		t1 = note_list_15[indexMusik];
		t2 = note_list_15[(indexMusik)+1];
	}
	if (cmpChanson == 16) {
		t1 = note_list_16[indexMusik];
		t2 = note_list_16[(indexMusik)+1];
	}
	if (cmpChanson == 17) {
		t1 = note_list_17[indexMusik];
		t2 = note_list_17[(indexMusik)+1];
	}
	if (cmpChanson == 18) {
		t1 = note_list_18[indexMusik];
		t2 = note_list_18[(indexMusik)+1];
	}
	if (cmpChanson == 19) {
		t1 = note_list_19[indexMusik];
		t2 = note_list_19[(indexMusik)+1];
	}

	if ((t1==0) && (t2==0)) {

		cmpChanson = random % NUMBER_OF_SONG;
		delayMusik50ms = 10;
		indexMusik = 0;
		if (cmpChanson>=NUMBER_OF_SONG)
			cmpChanson = 0;		
		return;
	}

	if (t1==0) {
		IO_HP_PLUS = 0;	
		SetDCPWM2(0);
	}
	else {
		if (mode_choc) {
			t1 = t1>>1;
			if (t1==0) t1 = 1;
		}
		IO_HP_PLUS = 1;	
		OpenPWM2(t1);
		SetDCPWM2(((int)t1)<<1);
	}
	delayMusik50ms = t2;
	indexMusik++;

}


// ************************
// ** Fonction principal **
// ************************

void main(void) {

	cmpChanson = 0;
	indexMusik = 0;
	delayMusik50ms = 0;
	cmpMusikLow = 0;
	mode_choc = 0;
	modeTourne = 0;

	// Convertisseur analogique/numerique actives
	OpenADC(ADC_FOSC_2 & ADC_1ANA_0REF & ADC_LEFT_JUST, ADC_CH0 & ADC_INT_OFF);
	ANStatus = 0;
	readADCOk = 0;
	ConvertADC();

	// Set PWM as output
	TRISCbits.TRISC1 = 0;
	TRISCbits.TRISC2 = 0;
	// Set motor direction constrol as output
	TRISCbits.TRISC0 = 0;
	TRISCbits.TRISC3 = 0;
	TRISCbits.TRISC4 = 0;
	TRISCbits.TRISC5 = 0;




	// TIMERS
	// Timer 1 -> as 16 bit counter
	//OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_1 & T1_OSC1EN_OFF & T1_SYNC_EXT_OFF);

 	// Timer 2 -> for the PWM and 0
	// 1ms of period for the PWM (255+1) * 16 (prescaler) * 1s = 4ms
	OpenTimer2(TIMER_INT_OFF & T2_PS_1_16 & T2_POST_1_1);
	// PWM
	OpenPWM1(249); // 249
	OpenPWM2(249); // 249
	SetDCPWM1(1001); // 500
	SetDCPWM2(1001); // 500

	// Timer 3 -> for servo
	OpenTimer3(TIMER_INT_ON & T3_16BIT_RW & T3_SOURCE_INT & T3_PS_1_1 & T3_SYNC_EXT_ON & T1_SOURCE_CCP);

	// Timer 0 -> as 8 bit
	OpenTimer0(TIMER_INT_ON & T0_8BIT & T0_SOURCE_INT & T0_PS_1_2);

	// Set the timer 0
	WriteTimer0(TIMER0_RESTART);
	mainCompteur = 0;
	mainCompteurLong = 0;

	// set I/O var
	servo1_pos = SERVO_MIDLE;
	servo2_pos = SERVO_MIDLE;
	servog_pos = 0;
	servod_pos = 0;
	servoStatus = 0;

	// I/O 
	// Set servo port as output
	IO_SERVO1_TRIS = 0;
	IO_SERVO2_TRIS = 0;
	IO_SERVOG_TRIS = 0;
	IO_SERVOD_TRIS = 0;
	IO_LED_TRIS = 0;
	
	// set led as output
	IO_LED_ROUGE_TRIS = 0;
	IO_LED_BLANCHE_TRIS = 0;

	// set hp as output
	IO_HP_PLUS = 0;
	IO_HP_MOINS = 0;
	IO_HP_PLUS_TRIS = 0;
	IO_HP_MOINS_TRIS = 0;

	// set IR as input
	IO_IR_AVANT_TRIS = 1;
	IO_IR_ARRIERE_TRIS = 1;
	IO_IR_GAUCHE_TRIS = 1;
	IO_IR_DROITE_TRIS = 1;

	// set CHOC dectection as input
	IO_CHOC_TRIS = 1;
	

	// Start
	EnableAllInterrupt();

	// Start servo
	setServo(1);

	servo1_pos = SERVO_M45;
	servo2_pos = SERVO_MIDLE;

	/* Gogo ! */
	robotAvance();
	newhalfms = 0;
	cmpAd = 0;

	OpenPWM2(142); // period
	SetDCPWM2(0);
	musique();



	/* boucle infinie */
	while (1) {

	/* test choc */ 
	if (IO_CHOC) {
		mode_choc = 1;
		cmpChoc = 0;
		choix_choc = random&1;
	}


	/* valeur dispo pour la distance */
	if (readADCOk) {	

		distanceNew = (uchar)(ReadADC()>>8);
		random += distanceNew;
		if (ReadADC() & 64)
			random++;

		if (distanceNew>126)
			distanceNew = 20;
		else	
			distanceNew = valueIR[distanceNew]; /* convert en 0.5cm */

		distanceArray[cmpAd] = distanceNew;

		//testIR();

		/* Calcule une moyenne en eleminant les parasites .. */
		cmpAd++;
		if (cmpAd>7) {
			
			sum = 0;
			for (cmpAd=0; cmpAd<8; cmpAd++) {
				sum += distanceArray[cmpAd];			
			}
			sum = sum>>3;		

			//testIR();

			distance =sum;
			cmpAd = 0;


			if (!modeTourne) {	
				ledCBlanche = 1+((distance-20)>>3);
				if (ledCBlanche>10)
					ledCBlanche = 10;
			}
			
			if (distance<60) { // 40
				IO_LED = 1;
			}
			else {
				IO_LED = 0;
			}


			/*			
			if (distance>110) {
				IO_LED = 1;
			}
			else {
				IO_LED = 0;
			}*/


			// ya un trou devant !
			if (distance>110)
				distance = 20;

			if (mode_choc) {
				ledCRouge = 10;
			} else {
				if (!modeTourne) {	
					if (distance==60) { /* 40 */
						servo2_pos = SERVO_MIDLE;
						ledCRouge = 10;
					}
					else {
						if (distance<60) { /* 40 */
							servo2_pos = SERVO_MIDLE-((SERVO_MIDLE-SERVO_M90)>>5)*(int)(60-distance);
							/*servo2_pos = SERVO_MIDLE-((SERVO_MIDLE-SERVO_M90)>>4)*(int)(40-distance);*/
							if (servo2_pos<SERVO_M90) {
								servo2_pos = SERVO_M90;
								/*ledCRouge = 10;*/
							} /*else {
								ledCRouge = 9-(uchar)((servo2_pos - SERVO_M90)>>7);
							}*/
							ledCRouge = 10;
			
						}
						else {
							ledCRouge = 5;
							if (distance>60) { /* 40 */
								if (stateTourelle==0) {
									/*servo2_pos = SERVO_MIDLE+((SERVO_P90-SERVO_MIDLE)>>7)*(int)(distance-40); */
									servo2_pos = SERVO_MIDLE+((SERVO_P90-SERVO_MIDLE)>>7)*(int)(distance-60);
									if (servo2_pos>SERVO_P90)
										servo2_pos = SERVO_P90;					
								}
								else
									servo2_pos = SERVO_MIDLE;
							}
						}
					}
				}
			}	

			//testIR();
		}

		readADCOk = 0;
	}

	/*
	tmp++;
	if (tmp&4)
		testIR();*/


	if (newhalfms) {

		cmpMusikLow++;
		if (cmpMusikLow == 100) {
			cmpMusikLow = 0;
			if (delayMusik50ms==0) {
				musique();
			}
			else {
				delayMusik50ms--;
				if (delayMusik50ms==0)
					musique();
			}
		}

		newhalfms = 0;

		cmpStartTourne++;
		if (cmpStartTourne == 30000) {
			cmpStartTourne = 0;
			if (random & 1) {
				modeTourne = 1;	
				cmpTourne = 0;
			}
		}



		if (mode_choc) {
			cmpChoc++;
			if (cmpChoc>4000) {
				cmpChoc = 0;
				mode_choc = 0;
			}
			else {
				if (cmpChoc<2000) {
					servo2_pos = SERVO_MIDLE;
					servo1_pos = SERVO_MIDLE;
					robotRecule();
				}
				else {
					if (choix_choc) {
						servo2_pos = SERVO_M90;
						servo1_pos = SERVO_P90;
						robotDroiteFort();
					}
					else {
						servo2_pos = SERVO_P90;
						servo1_pos = SERVO_M90;
						robotGaucheFort();
					}
				}
			}
		}
		else {

			if (modeTourne) {

				cmpLedTourne++;
				if (cmpLedTourne==500) {
					cmpLedTourne = 0;
					if (ledCRouge==0) {
						ledCRouge = 10;
						ledCBlanche = 10;
					} else {
						ledCRouge = 0;
						ledCBlanche = 0;
					}
				}

				cmpTourne++;
				if (cmpTourne>24000) {
					modeTourne = 0;
					cmpTourne = 0;
					cmpStartTourne = 0;
				} else {			
					if (cmpTourne<2000) {
						ledCRouge = 10;
						ledCBlanche = 10;
						servo1_pos = SERVO_MIDLE;
						servo2_pos = SERVO_MIDLE;
						robotStop();
					} else {
						if (cmpTourne<4000) {
							servo1_pos = SERVO_MIDLE;
							servo2_pos = SERVO_MIDLE;
							robotRecule();
						} else {
							if (cmpTourne<5000) {
								servo1_pos = SERVO_MIDLE;
								servo2_pos = SERVO_P90;
								robotGaucheFort();								
							} else {
								if (cmpTourne<7000) {
									servo1_pos = SERVO_MIDLE;
									servo2_pos = SERVO_MIDLE;
									robotAvance();
								} else {
									if (cmpTourne<15000) {
										servo1_pos = SERVO_MIDLE;
										servo2_pos = SERVO_M90;
										robotDroiteFort();
									} else {
										if (cmpTourne<23000) {
											servo1_pos = SERVO_MIDLE;
											servo2_pos = SERVO_P90;
											robotGaucheFort();
										} else {
											ledCRouge = 10;
											ledCBlanche = 10;
											servo1_pos = SERVO_MIDLE;
											servo2_pos = SERVO_MIDLE;
											robotStop();
										}
									}
								}
							}
						}
					}
				}
			}
			else {
				if (servo2_pos>(SERVO_MIDLE-318)) {
		
					cmpStateTourelle++;
					if ((stateTourelle ==0) && (cmpStateTourelle>2000)) {
						cmpStateTourelle = 0;
						stateTourelle = 1;	
						servo1_pos = SERVO_MIDLE;
					}
					if ((stateTourelle !=0) && (cmpStateTourelle>700)) {
						cmpStateTourelle = 0;
						stateTourelle++;
						if ((stateTourelle==3)) {
							stateTourelle = 0;
							servo1_pos = SERVO_M45;
						}
						if (stateTourelle==2) {
							servo1_pos = SERVO_P45;
						}
					}
				} else
					cmpStateTourelle = 0;
		
				//testIR();

			
				if (servo2_pos<SERVO_MIDLE-318) {
					if (servo2_pos<SERVO_M90+212) {
						typeAvance = 2; /*robotDroiteFort();*/
					} else {
						typeAvance = 1; /*robotDroite(); */
					}
				} else {
					if (servo2_pos>SERVO_MIDLE+318) {
						if (servo2_pos<SERVO_P90-212) {
							typeAvance = 4; /*robotGaucheFort(); */
						} else {
							typeAvance = 3; /*robotGauche();*/
						}
					}
					else
					typeAvance = 0; /* robotAvance(); */
				}
		
				//testIR();
			
				/* gere un delais de changement pour eviter les acoups */
				if (typeAvance == oldTypeAvance) {
					cmpTypeAvance++;
					if (cmpTypeAvance>200) {
						cmpTypeAvance = 200;				
						switch (typeAvance) 
						{
							case 0:
								robotAvance();
								break;
							case 1:
								robotDroite();
								break;
							case 2:
								robotDroiteFort();
								break;
							case 3:
								robotGauche();
								break;
							case 4:
								robotGaucheFort();
								break;
						}			
					}
				}
				else {
					cmpTypeAvance = 0;
					oldTypeAvance = typeAvance;
				}
			}
		}

		//testIR();
	}
	
		
		if (mainCompteurLong>50) {
			mainCompteurLong = 0;

		}

	}	
}

// Configuration du PIC
// ne pas mettre de commentaires dans cette macro !!!
// Code protect zone de boot active
#pragma romdata CONFIG
_CONFIG_DECL (
_CONFIG1H_DEFAULT & _OSC_HS_1H , 
_CONFIG2L_DEFAULT & _PWRT_ON_2L , 
_CONFIG2H_DEFAULT & _WDT_OFF_2H,
_CONFIG3H_DEFAULT,
_CONFIG4L_DEFAULT & _STVR_OFF_4L & _LVP_OFF_4L & _DEBUG_OFF_4L ,
_CONFIG5L_DEFAULT & _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L,
_CONFIG5H_DEFAULT & _CPB_OFF_5H & _CPD_OFF_5H,
_CONFIG6L_DEFAULT & _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L,
_CONFIG6H_DEFAULT & _WPC_OFF_6H & _WPB_ON_6H & _WPD_OFF_6H,
_CONFIG7L_DEFAULT & _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L,
_CONFIG7H_DEFAULT & _EBTRB_OFF_7H
); 
#pragma romdata
